package com.altimetrik.pmis.controller;


import com.altimetrik.pmis.model.CreateProjectSummaryRequest;
import com.altimetrik.pmis.model.ProjectSummary;
import com.altimetrik.pmis.model.WeeklyMetricReport;
import com.altimetrik.pmis.service.ExcelService;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.time.LocalDate;
import java.util.List;

@RestController
@RequestMapping("/excel")
@Validated
public class ExcelController {

    private ExcelService excelService;

    @Autowired
    public ExcelController(ExcelService excelService) {
        this.excelService = excelService;
    }

    @PostMapping(value = "/download-excel/project-summary", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> downloadProjectSummaryExcel(@Valid @RequestBody CreateProjectSummaryRequest createProjectSummaryRequest) throws IOException {
        byte[] body = excelService.downloadProjectSummaryExcel(createProjectSummaryRequest);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
        headers.setContentDispositionFormData("attachment", createProjectSummaryRequest.getAccountName() + "-account-project-summary.xlsx");

        return ResponseEntity.ok()
                .headers(headers)
                .body(body);

    }

    @PostMapping("/upload-excel/project-summary")
    public ResponseEntity<List<ProjectSummary>> uploadProjectSummaryExcel(@RequestParam("file") MultipartFile file) throws IOException {
        List<ProjectSummary> projectSummaries = excelService.uploadProjectSummaryExcel(file);

        return ResponseEntity.ok(projectSummaries);
    }

    @PostMapping(value = "/download-excel/weekly-report/account/{accountId}/project/{projectId}/reportDate/{reportDate}", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    public ResponseEntity<byte[]> downloadWeeklyReportExcel(@PathVariable("accountId") Long accountId,
                                                            @PathVariable("projectId") Long projectId,
                                                            @PathVariable("reportDate") @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate reportDate) throws IOException {
        byte[] body = excelService.downloadWeeklyReportExcel(accountId, projectId, reportDate);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_OCTET_STREAM);
        headers.setContentDispositionFormData("attachment", "weekly_report.xlsx");

        return ResponseEntity.ok()
                .headers(headers)
                .body(body);

    }

    @PostMapping("/upload-excel/weekly-report")
    public ResponseEntity<WeeklyMetricReport> uploadWeeklyReportExcel(@RequestParam("file") MultipartFile file) throws IOException {
        WeeklyMetricReport weeklyMetricReport = excelService.uploadWeeklyReportExcel(file);

        return ResponseEntity.ok(weeklyMetricReport);
    }
}
