package com.altimetrik.pmis.controller;

import com.altimetrik.pmis.model.Metric;
import com.altimetrik.pmis.model.WeeklyMetricReport;
import com.altimetrik.pmis.service.MetricsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.util.Objects;

@RestController
@RequestMapping("/metrics")
public class MetricsController {
    private final MetricsService metricsService;

    @Autowired
    private MetricsController(MetricsService metricsService) {
        this.metricsService = metricsService;
    }

    @PostMapping("/account")
    public ResponseEntity<Metric> saveAccountMetric(@RequestBody Metric metric) {
        return ResponseEntity.ok(metricsService.saveAccountMetric(metric));
    }

    @PostMapping("/project")
    public ResponseEntity<Metric> saveProjectMetric(@RequestBody Metric metric) {
        return ResponseEntity.ok(metricsService.saveProjectMetric(metric));
    }

    @GetMapping("account/{accountId}")
    public ResponseEntity<Metric> getAccountMetricsByAccountId(@PathVariable("accountId") Long accountId) {
        Metric metric = metricsService.getAccountMetricsByAccountId(accountId);
        if (Objects.isNull(metric)) {
            return ResponseEntity.noContent().build();
        }
        return ResponseEntity.ok(metric);
    }

    @GetMapping("project/{projectId}")
    public ResponseEntity<Metric> getProjectMetricsByProjectId(@PathVariable("projectId") Long projectId) {
        Metric metric = metricsService.getProjectMetricsByProjectId(projectId);
        if (Objects.isNull(metric)) {
            return ResponseEntity.noContent().build();
        }
        return ResponseEntity.ok(metric);
    }

    @GetMapping("/weekly-report/account/{accountId}/project/{projectId}/reportDate/{reportDate}")
    public ResponseEntity<WeeklyMetricReport> getProjectMetricsByProjectId(@PathVariable("accountId") Long accountId,
                                                                           @PathVariable("projectId") Long projectId,
                                                                           @PathVariable("reportDate") @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate reportDate) {
        WeeklyMetricReport weeklyMetricReport = metricsService.getOrCreateWeeklyMetricReportForProject(accountId, projectId, reportDate);

        return ResponseEntity.ok(weeklyMetricReport);
    }

    @PutMapping("/weekly-report")
    public ResponseEntity<WeeklyMetricReport> updateWeeklyReportForProject(@RequestBody WeeklyMetricReport weeklyMetricReport) {
        WeeklyMetricReport weeklyMetricReportResponse = metricsService.updateWeeklyReportForProject(weeklyMetricReport);

        return ResponseEntity.ok(weeklyMetricReportResponse);
    }

}
