package com.altimetrik.pmis.controller;

import com.altimetrik.pmis.model.CreateProjectSummaryRequest;
import com.altimetrik.pmis.model.Project;
import com.altimetrik.pmis.model.ProjectSummary;
import com.altimetrik.pmis.model.ProjectSummaryResponse;
import com.altimetrik.pmis.service.ProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDate;
import java.util.List;

@RestController
@RequestMapping("/projects")
public class ProjectController {
    private final ProjectService projectService;

    @Autowired
    public ProjectController(ProjectService projectService) {
        this.projectService = projectService;
    }

    @PostMapping("/account/{accountId}")
    public ResponseEntity<Project> save(@PathVariable(name = "accountId") Long accountId, @RequestBody Project project) {
        return ResponseEntity.ok(projectService.save(accountId, project));
    }

    @GetMapping("/{id}")
    public ResponseEntity<Project> get(@PathVariable Long id) {
        return ResponseEntity.ok(projectService.get(id));
    }

    @GetMapping("/account/{accountId}")
    public ResponseEntity<List<Project>> getByAccount(@PathVariable Long accountId) {
        return ResponseEntity.ok(projectService.getByAccount(accountId));
    }

    @PostMapping("/summary")
    public ResponseEntity<ProjectSummaryResponse> createProjectSummary(@RequestBody CreateProjectSummaryRequest createProjectSummaryRequest) {
        return ResponseEntity.ok(projectService.createProjectSummary(createProjectSummaryRequest));
    }

    @GetMapping("/summary/account/{accountId}/reportDate/{reportDate}")
    public ResponseEntity<ProjectSummaryResponse> getProjectSummaryByAccountAndReportDate(@PathVariable("accountId") Long accountId, @PathVariable("reportDate") @DateTimeFormat(pattern = "yyyy-MM-dd") LocalDate reportDate) {
        return ResponseEntity.ok(projectService.getProjectSummaryByAccountAndReportDate(accountId, null, reportDate));
    }

    @PutMapping("/summary")
    public ResponseEntity<ProjectSummary> updateProjectSummary(@RequestBody ProjectSummary projectSummary) {
        return ResponseEntity.ok(projectService.updateProjectSummary(projectSummary));
    }

    @PutMapping("/summary/multiple")
    public ResponseEntity<List<ProjectSummary>> updateMultipleProjectSummary(@RequestBody List<ProjectSummary> projectSummaries) {
        return ResponseEntity.ok(projectService.updateMultipleProjectSummary(projectSummaries));
    }
}
