package com.altimetrik.pmis.mapper;

import com.altimetrik.pmis.core.entity.*;
import com.altimetrik.pmis.core.enums.Status;
import com.altimetrik.pmis.model.Metric;
import com.altimetrik.pmis.model.MetricIndicator;
import com.altimetrik.pmis.model.WeeklyMetric;
import com.altimetrik.pmis.model.WeeklyMetricReport;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.factory.Mappers;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

@Mapper
public interface MetricsMapper {
    MetricsMapper INSTANCE = Mappers.getMapper(MetricsMapper.class);

    private static WeeklyMetric mapMetricIndicatorsDomainToWeeklyMetricIndicatorsDomain(Long projectId, MetricIndicator metricIndicator, boolean isAccountMetricIndicator) {
        return WeeklyMetric.builder()
                .id(null)
                .accountMetricId(isAccountMetricIndicator ? metricIndicator.getMetricId() : null)
                .projectMetricId(!isAccountMetricIndicator ? metricIndicator.getMetricId() : null)
//                .projectId(projectId)
                .categoryType(metricIndicator.getCategoryType())
                .categoryName(metricIndicator.getCategoryName())
                .indicatorName(metricIndicator.getIndicatorName())
                .build();
    }

    private static void mapIndicatorToWeeklyReportDomain(IndicatorEntity indicatorEntity, WeeklyMetric weeklyMetric) {
        weeklyMetric.setCategoryName(indicatorEntity.getCategory().getCategoryName());
        weeklyMetric.setCategoryType(indicatorEntity.getCategory().getType());
        weeklyMetric.setIndicatorName(indicatorEntity.getName());
    }

    default Metric accountMetricEntitiesToDomain(List<AccountMetricEntity> accountMetricEntities) {
        Metric metric = null;
        for (AccountMetricEntity accountMetricEntity : accountMetricEntities) {
            if (Objects.isNull(metric)) {
                metric = accountMetricEntityToDomain(accountMetricEntity);
            }
            metric.getMetricIndicators().add(mapIndicatorEntityToDomain(accountMetricEntity.getId(), accountMetricEntity.getIndicator(), accountMetricEntity.isActive()));
        }

        return metric;
    }

    default List<AccountMetricEntity> domainToEntity(Metric metric, AccountEntity accountEntity, Optional<List<IndicatorEntity>> indicatorEntitiesOptional) {
        List<AccountMetricEntity> accountMetricEntities = new ArrayList<>();
        if (indicatorEntitiesOptional.isPresent()) {
            accountMetricEntities = indicatorEntitiesOptional.get().stream()
                    .map((indicatorEntity) -> {
                        Optional<MetricIndicator> metricIndicatorOptional = metric.getMetricIndicators().stream().filter(metricIndicator -> metricIndicator.getIndicatorId().equals(indicatorEntity.getId())).findFirst();

                        return AccountMetricEntity.builder()
                                .id(null)
                                .account(accountEntity)
                                .indicator(indicatorEntity)
                                .isActive(metricIndicatorOptional.isPresent() ? metricIndicatorOptional.get().isActive() : false)
                                .build();
                    })
                    .collect(Collectors.toList());
        }
        return accountMetricEntities;
    }

    @Mapping(target = "accountId", source = "account.id")
    Metric accountMetricEntityToDomain(AccountMetricEntity accountMetricEntity);

    @Mapping(target = "active", source = "isActive")
    @Mapping(target = "metricId", source = "metricId")
    @Mapping(target = "indicatorId", source = "indicatorEntity.id")
    @Mapping(target = "indicatorName", source = "indicatorEntity.name")
    @Mapping(target = "categoryType", source = "indicatorEntity.category.type")
    @Mapping(target = "categoryName", source = "indicatorEntity.category.categoryName")
    @Mapping(target = "description", source = "indicatorEntity.category.categoryName")
    MetricIndicator mapIndicatorEntityToDomain(Long metricId, IndicatorEntity indicatorEntity, boolean isActive);

    @Mapping(target = "projectId", source = "project.id")
    @Mapping(target = "accountId", source = "project.account.id")
    Metric projectMetricEntityToDomain(ProjectMetricEntity projectMetricEntity);

    default Metric projectMetricEntitiesToDomain(List<ProjectMetricEntity> projectMetricEntities) {
        Metric metric = null;
        for (ProjectMetricEntity projectMetricEntity : projectMetricEntities) {
            if (Objects.isNull(metric)) {
                metric = projectMetricEntityToDomain(projectMetricEntity);
            }
            metric.getMetricIndicators().add(mapIndicatorEntityToDomain(projectMetricEntity.getId(), projectMetricEntity.getIndicator(), projectMetricEntity.isActive()));
        }

        return metric;
    }

    default List<ProjectMetricEntity> domainToEntity(Metric metric, ProjectEntity projectEntity, Optional<List<IndicatorEntity>> indicatorEntitiesOptional) {
        List<ProjectMetricEntity> projectMetricEntities = new ArrayList<>();
        if (indicatorEntitiesOptional.isPresent()) {
            projectMetricEntities = indicatorEntitiesOptional.get().stream()
                    .map((indicatorEntity) -> {
                        Optional<MetricIndicator> metricIndicatorOptional = metric.getMetricIndicators().stream().filter(metricIndicator -> metricIndicator.getIndicatorId().equals(indicatorEntity.getId())).findFirst();

                        return ProjectMetricEntity.builder()
                                .id(null)
                                .project(projectEntity)
                                .indicator(indicatorEntity)
                                .isActive(metricIndicatorOptional.isPresent() ? metricIndicatorOptional.get().isActive() : false)
                                .build();
                    })
                    .collect(Collectors.toList());
        }
        return projectMetricEntities;

    }

    default WeeklyMetricReport mapMetricDomainToWeeklyMetricReportDomain(Metric accountMetric, Metric projectMetric, Long projectId, LocalDate reportDate) {
        WeeklyMetricReport weeklyMetricReport = new WeeklyMetricReport();
        weeklyMetricReport.setReportDate(reportDate);
        weeklyMetricReport.setProjectId(projectId);
        List<WeeklyMetric> weeklyMetrics = null;
        if (Objects.nonNull(accountMetric) && !CollectionUtils.isEmpty(accountMetric.getMetricIndicators())) {
            weeklyMetrics = accountMetric.getMetricIndicators().stream()
                    .map(am -> {
                        return mapMetricIndicatorsDomainToWeeklyMetricIndicatorsDomain(projectId, am, true);
                    }).collect(Collectors.toList());
            if (Objects.nonNull(projectMetric) && !CollectionUtils.isEmpty(projectMetric.getMetricIndicators())) {
                weeklyMetrics.addAll(projectMetric.getMetricIndicators().stream()
                        .map(pm -> {
                            return mapMetricIndicatorsDomainToWeeklyMetricIndicatorsDomain(projectId, pm, false);
                        }).collect(Collectors.toList()));
            }
        }
        weeklyMetricReport.setWeeklyMetrics(weeklyMetrics);

        return weeklyMetricReport;
    }

    default List<WeeklyMetricEntity> mapWeeklyMetricEntity(List<AccountMetricEntity> accountMetricEntities,
                                                           List<ProjectMetricEntity> projectMetricEntities,
                                                           TrendEntity trendEntity) {
        List<WeeklyMetricEntity> weeklyMetricEntities = null;
        if (!CollectionUtils.isEmpty(accountMetricEntities)) {
            weeklyMetricEntities = accountMetricEntities.stream()
                    .map(accountMetricEntity -> {
                        WeeklyMetricEntity weeklyMetricEntity = new WeeklyMetricEntity();
                        weeklyMetricEntity.setAccountMetric(accountMetricEntity);
                        weeklyMetricEntity.setReportDate(trendEntity.getReportDate());
                        weeklyMetricEntity.setProject(trendEntity.getProject());
                        weeklyMetricEntity.setTrendEntity(trendEntity);
                        return weeklyMetricEntity;
                    })
                    .collect(Collectors.toList());
        }
        if (!CollectionUtils.isEmpty(projectMetricEntities)) {
            weeklyMetricEntities.addAll(projectMetricEntities.stream()
                    .map(projectMetricEntity -> {
                        WeeklyMetricEntity weeklyMetricEntity = new WeeklyMetricEntity();
                        weeklyMetricEntity.setProjectMetric(projectMetricEntity);
                        weeklyMetricEntity.setProject(trendEntity.getProject());
                        weeklyMetricEntity.setTrendEntity(trendEntity);
                        return weeklyMetricEntity;
                    })
                    .collect(Collectors.toList()));
        }

        return weeklyMetricEntities;
    }

    WeeklyMetricEntity weeklyMetricToWeeklyMetricEntity(WeeklyMetric weeklyMetric);

    default WeeklyMetricReport mapWeeklyMetricEntityToWeeklyMetricReport(List<WeeklyMetricEntity> weeklyMetricEntities, LocalDate reportDate, List<Status> weeklyTrend) {
        WeeklyMetricReport weeklyMetricReport = new WeeklyMetricReport();
        weeklyMetricReport.setReportDate(reportDate);
        weeklyMetricReport.setWeeklyTrends(weeklyTrend);

        weeklyMetricReport.setWeeklyMetrics(weeklyMetricEntities.stream()
                .map(weeklyMetricEntity -> {
                    WeeklyMetric weeklyMetric = weeklyMetricEntityToWeeklyMetricDomain(weeklyMetricEntity);
                    if (Objects.nonNull(weeklyMetricEntity.getAccountMetric()) && Objects.nonNull(weeklyMetricEntity.getAccountMetric().getIndicator())) {
                        mapIndicatorToWeeklyReportDomain(weeklyMetricEntity.getAccountMetric().getIndicator(), weeklyMetric);
                    } else if (Objects.nonNull(weeklyMetricEntity.getProjectMetric()) && Objects.nonNull(weeklyMetricEntity.getProjectMetric().getIndicator())) {
                        mapIndicatorToWeeklyReportDomain(weeklyMetricEntity.getProjectMetric().getIndicator(), weeklyMetric);
                    }
                    weeklyMetricReport.setTrendId(Objects.nonNull(weeklyMetricEntity.getTrendEntity()) ? weeklyMetricEntity.getTrendEntity().getId() : null);
                    if (Objects.nonNull(weeklyMetricEntity.getProject())) {
                        weeklyMetricReport.setProjectId(weeklyMetricEntity.getProject().getId());
                        weeklyMetricReport.setProjectName(weeklyMetricEntity.getProject().getName());
                    }

                    return weeklyMetric;
                }).collect(Collectors.toList()));

        return weeklyMetricReport;

    }


    @Mapping(target = "projectMetricId", source = "projectMetric.id")
    @Mapping(target = "accountMetricId", source = "accountMetric.id")
    WeeklyMetric weeklyMetricEntityToWeeklyMetricDomain(WeeklyMetricEntity weeklyMetricEntity);

    default List<WeeklyMetricEntity> mapWeeklyMetricReportToWeeklyMetricEntity(WeeklyMetricReport weeklyMetricReport, TrendEntity trendEntity) {
        List<WeeklyMetricEntity> weeklyMetricEntities = weeklyMetricReport.getWeeklyMetrics().stream()
                .map(weeklyMetric -> {
                    WeeklyMetricEntity weeklyMetricEntity = weeklyMetricToWeeklyMetricEntity(weeklyMetric);
                    weeklyMetricEntity.setProject(ProjectEntity.builder().id(weeklyMetricReport.getProjectId()).build());
                    weeklyMetricEntity.setAccountMetric(Objects.nonNull(weeklyMetric.getAccountMetricId()) ? AccountMetricEntity.builder().id(weeklyMetric.getAccountMetricId()).build() : null);
                    weeklyMetricEntity.setProjectMetric(Objects.nonNull(weeklyMetric.getProjectMetricId()) ? ProjectMetricEntity.builder().id(weeklyMetric.getProjectMetricId()).build() : null);
                    weeklyMetricEntity.setReportDate(weeklyMetricReport.getReportDate());
                    weeklyMetricEntity.setTrendEntity(trendEntity);
                    return weeklyMetricEntity;
                }).collect(Collectors.toList());
        return weeklyMetricEntities;

    }
}
