package com.altimetrik.pmis.service.impl;

import com.altimetrik.pmis.core.common.Constants;
import com.altimetrik.pmis.core.enums.CategoryType;
import com.altimetrik.pmis.core.enums.Status;
import com.altimetrik.pmis.model.*;
import com.altimetrik.pmis.service.ExcelService;
import com.altimetrik.pmis.service.MetricsService;
import com.altimetrik.pmis.service.ProjectService;
import com.altimetrik.pmis.util.StringToDataTypeConverter;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
public class ExcelServiceImpl implements ExcelService {

    private ProjectService projectService;

    private MetricsService metricsService;

    @Autowired
    public ExcelServiceImpl(ProjectService projectService, MetricsService metricsService) {
        this.projectService = projectService;
        this.metricsService = metricsService;
    }

    @Override
    public byte[] downloadProjectSummaryExcel(CreateProjectSummaryRequest createProjectSummaryRequest) throws IOException {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try (Workbook workbook = new XSSFWorkbook()) {
            Sheet sheet = workbook.createSheet("Sheet1");


            Font columnFont = workbook.createFont();
            columnFont.setBold(true);

            CellStyle lockedAccountHeaderColumnCellStyle = workbook.createCellStyle();
            lockedAccountHeaderColumnCellStyle.setAlignment(HorizontalAlignment.LEFT);
            lockedAccountHeaderColumnCellStyle.setFont(columnFont);
            lockedAccountHeaderColumnCellStyle.setWrapText(true);

            CellStyle lockedColumnCellStyle = workbook.createCellStyle();
            lockedColumnCellStyle.setAlignment(HorizontalAlignment.CENTER_SELECTION);
            lockedColumnCellStyle.setFont(columnFont);


            ProjectSummaryResponse projectSummaryResponse = projectService.createProjectSummary(createProjectSummaryRequest);

            if (Objects.nonNull(projectSummaryResponse)) {
                Row accountNameRow = sheet.createRow(Constants.PS_EXCEL_ACCOUNT_NAME_ROW);

                Cell accountIdCell = accountNameRow.createCell(Constants.PS_EXCEL_ACCOUNT_ID_CELL);
                accountIdCell.setCellValue(projectSummaryResponse.getAccountId());

                Cell accountNameTitleCell = accountNameRow.createCell(Constants.PS_EXCEL_ACCOUNT_NAME_TITLE_CELL);
                accountNameTitleCell.setCellValue(Constants.PS_EXCEL_ACCOUNT_NAME_TITLE);
                accountNameTitleCell.setCellStyle(lockedAccountHeaderColumnCellStyle);

                Cell accountNameValueCell = accountNameRow.createCell(Constants.PS_EXCEL_ACCOUNT_NAME_VALUE_CELL);
                accountNameValueCell.setCellValue(projectSummaryResponse.getAccountName());

                Row reportDateRow = sheet.createRow(Constants.PS_EXCEL_REPORT_DATE_ROW);
                Cell reportDateTitleCell = reportDateRow.createCell(Constants.PS_EXCEL_REPORT_DATE_TITLE_CELL);
                reportDateTitleCell.setCellValue(Constants.PS_EXCEL_REPORT_DATE_TITLE);
                reportDateTitleCell.setCellStyle(lockedAccountHeaderColumnCellStyle);

                Cell reportDateValueCell = reportDateRow.createCell(Constants.PS_EXCEL_REPORT_DATE_VALUE_CELL);
                reportDateValueCell.setCellValue(projectSummaryResponse.getReportDate());
                CellStyle reportDateValueCellStyle = workbook.createCellStyle();
                CreationHelper creationHelper = workbook.getCreationHelper();
                short dateFormat = creationHelper.createDataFormat().getFormat("yyyy-MM-dd");
                reportDateValueCellStyle.setDataFormat(dateFormat);
                reportDateValueCell.setCellStyle(reportDateValueCellStyle);

                String[] headersRow1 = Constants.PS_EXCEL_HEADER1_TITLES;
                Row headerRow1 = sheet.createRow(Constants.PS_EXCEL_HEADER1_ROW);
                headerRow1.setRowStyle(lockedColumnCellStyle);
                for (int i = 0; i < headersRow1.length; i++) {
                    Cell cell = headerRow1.createCell(i);
                    cell.setCellValue(headersRow1[i]);
                }
                String[] headersRow2 = Constants.PS_EXCEL_HEADER2_TITLES;
                Row headerRow2 = sheet.createRow(Constants.PS_EXCEL_HEADER2_ROW);
                headerRow2.setRowStyle(lockedColumnCellStyle);
                for (int i = 0; i < headersRow2.length; i++) {
                    Cell cell = headerRow2.createCell(i);
                    cell.setCellValue(headersRow2[i]);
                }
                sheet.addMergedRegion(new CellRangeAddress(Constants.PS_EXCEL_HEADER1_ROW, Constants.PS_EXCEL_HEADER1_ROW, 3, 6)); //merging PEOPLE header column
                sheet.addMergedRegion(new CellRangeAddress(Constants.PS_EXCEL_ACCOUNT_NAME_ROW, Constants.PS_EXCEL_ACCOUNT_NAME_ROW, Constants.PS_EXCEL_ACCOUNT_NAME_VALUE_CELL, 5)); //merging PEOPLE header column

                if (!CollectionUtils.isEmpty(projectSummaryResponse.getProjectSummaries())) {
                    int projectSummaryRowIndex = Constants.PS_EXCEL_HEADER2_ROW + 1;
                    for (int rowIndex = 0; rowIndex < projectSummaryResponse.getProjectSummaries().size(); rowIndex++, projectSummaryRowIndex++) {
                        Row row = sheet.createRow(projectSummaryRowIndex);

                        ProjectSummary projectSummary = projectSummaryResponse.getProjectSummaries().get(rowIndex);

                        //project id
                        Cell summaryIdCell = row.createCell(Constants.PS_EXCEL_SUMMARY_ID_VALUE_CELL, CellType.STRING);
                        summaryIdCell.setCellValue(String.valueOf(projectSummary.getId()));

                        //project id
                        Cell projectIdCell = row.createCell(Constants.PS_EXCEL_PROJECT_ID_VALUE_CELL, CellType.STRING);
                        projectIdCell.setCellValue(String.valueOf(projectSummary.getProjectId()));

                        //project name
                        Cell projectNameCell = row.createCell(Constants.PS_EXCEL_PROJECT_NAME_VALUE_CELL, CellType.STRING);
                        projectNameCell.setCellValue(projectSummary.getProjectName());
                        CellStyle projectNameValueCellStyle = workbook.createCellStyle();
                        projectNameCell.setCellStyle(projectNameValueCellStyle);

                        //engineering status
                        Cell engineeringStatusCell = row.createCell(Constants.PS_EXCEL_ENGINEERING_STATUS_VALUE_CELL, CellType.STRING);
                        engineeringStatusCell.setCellValue(Objects.nonNull(projectSummary.getEngineeringStatus())
                                ? projectSummary.getEngineeringStatus().name() : "NA");

                        //people planned
                        Cell peoplePlannedCell = row.createCell(Constants.PS_EXCEL_PEOPLE_PLANNED_VALUE_CELL, CellType.STRING);
                        peoplePlannedCell.setCellValue(String.valueOf(projectSummary.getPlanned()));

                        //people actual
                        Cell peopleActualCell = row.createCell(Constants.PS_EXCEL_PEOPLE_ACTUAL_VALUE_CELL, CellType.STRING);
                        peopleActualCell.setCellValue(String.valueOf(projectSummary.getActual()));

                        //people open
                        Cell peopleOpenCell = row.createCell(Constants.PS_EXCEL_PEOPLE_OPEN_VALUE_CELL, CellType.STRING);
                        peopleOpenCell.setCellValue(String.valueOf(projectSummary.getPlanned() - projectSummary.getActual()));

                        //people status
                        Cell peopleStatusCell = row.createCell(Constants.PS_EXCEL_PEOPLE_STATUS_VALUE_CELL, CellType.STRING);
                        peopleStatusCell.setCellValue(Objects.nonNull(projectSummary.getPeopleStatus())
                                ? projectSummary.getPeopleStatus().name() : "NA");

                        //remarks
                        Cell remarksCell = row.createCell(Constants.PS_EXCEL_REMARKS_VALUE_CELL, CellType.STRING);
                        remarksCell.setCellValue(StringUtils.isNotEmpty(projectSummary.getRemarks())
                                ? projectSummary.getRemarks().replace("|", "\n") : "");
                    }
                }
                sheet.setColumnHidden(0, true);
                sheet.setColumnHidden(1, true);
                // Set auto column width
                for (int columnIndex = 0; columnIndex <= Constants.PS_EXCEL_REMARKS_VALUE_CELL; columnIndex++) {
                    sheet.autoSizeColumn(columnIndex);
                }
                workbook.write(outputStream);
            }

        } catch (Exception ex) {
            throw ex;
        }
        return outputStream.toByteArray();
    }

    @Override
    public List<ProjectSummary> uploadProjectSummaryExcel(MultipartFile file) throws IOException {
        List<ProjectSummary> projectSummaries = new ArrayList<>();
        try (InputStream inputStream = file.getInputStream()) {

            Workbook workbook = new XSSFWorkbook(inputStream);
            Sheet sheet = workbook.getSheetAt(0);
            //ReportDate row
            Row reportDateRow = sheet.getRow(Constants.PS_EXCEL_REPORT_DATE_ROW);
            LocalDate reportDate = reportDateRow.getCell(Constants.PS_EXCEL_REPORT_DATE_VALUE_CELL).getLocalDateTimeCellValue().toLocalDate();
            int lastRowNum = sheet.getLastRowNum();
            int projectSummaryRowIndex = Constants.PS_EXCEL_HEADER2_ROW + 1;
            for (int i = projectSummaryRowIndex; i <= lastRowNum; i++) {
                Row row = sheet.getRow(i);
                if (Objects.nonNull(row)) {
                    projectSummaries.add(
                            ProjectSummary.builder()
                                    .id((Long) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.PS_EXCEL_SUMMARY_ID_VALUE_CELL).getStringCellValue(), Constants.LONG))
                                    .projectId((Long) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.PS_EXCEL_PROJECT_ID_VALUE_CELL).getStringCellValue(), Constants.LONG))
                                    .projectName(row.getCell(Constants.PS_EXCEL_PROJECT_ID_VALUE_CELL).getStringCellValue())
                                    .remarks(delimitRemarks(row.getCell(Constants.PS_EXCEL_REMARKS_VALUE_CELL).getStringCellValue()))
                                    .engineeringStatus(Status.valueOf(row.getCell(Constants.PS_EXCEL_ENGINEERING_STATUS_VALUE_CELL).getStringCellValue()))
                                    .planned((Integer) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.PS_EXCEL_PEOPLE_PLANNED_VALUE_CELL).getStringCellValue(), Constants.INT))
                                    .actual((Integer) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.PS_EXCEL_PEOPLE_ACTUAL_VALUE_CELL).getStringCellValue(), Constants.INT))
                                    .peopleStatus(Status.valueOf(row.getCell(Constants.PS_EXCEL_PEOPLE_STATUS_VALUE_CELL).getStringCellValue()))
                                    .reportDate(reportDate)
                                    .build()
                    );
                }
            }
        }
        if (!CollectionUtils.isEmpty(projectSummaries)) {
            return projectService.updateMultipleProjectSummary(projectSummaries);
        }

        return null;
    }

    @Override
    public byte[] downloadWeeklyReportExcel(Long accountId, Long projectId, LocalDate reportDate) throws IOException {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try (Workbook workbook = new XSSFWorkbook()) {
            Sheet sheet = workbook.createSheet("Sheet1");


            Font columnFont = workbook.createFont();
            columnFont.setBold(true);

            CellStyle lockedProjectHeaderColumnCellStyle = workbook.createCellStyle();
            lockedProjectHeaderColumnCellStyle.setAlignment(HorizontalAlignment.LEFT);
            lockedProjectHeaderColumnCellStyle.setFont(columnFont);
            lockedProjectHeaderColumnCellStyle.setWrapText(true);

            CellStyle lockedColumnCellStyle = workbook.createCellStyle();
            lockedColumnCellStyle.setAlignment(HorizontalAlignment.CENTER_SELECTION);
            lockedColumnCellStyle.setFont(columnFont);


            WeeklyMetricReport weeklyMetricReport = metricsService.getOrCreateWeeklyMetricReportForProject(accountId, projectId, reportDate);

            if (Objects.nonNull(weeklyMetricReport)) {
                Row projectNameRow = sheet.createRow(Constants.WR_EXCEL_PROJECT_NAME_ROW);

                Cell projectIdCell = projectNameRow.createCell(Constants.WR_EXCEL_PROJECT_ID_CELL);
                projectIdCell.setCellValue(String.valueOf(weeklyMetricReport.getProjectId()));

                Cell projectNameTitleCell = projectNameRow.createCell(Constants.WR_EXCEL_PROJECT_NAME_TITLE_CELL);
                projectNameTitleCell.setCellValue(Constants.WR_EXCEL_PROJECT_NAME_TITLE);
                projectNameTitleCell.setCellStyle(lockedProjectHeaderColumnCellStyle);

                Cell projectNameValueCell = projectNameRow.createCell(Constants.WR_EXCEL_PROJECT_NAME_VALUE_CELL);
                projectNameValueCell.setCellValue(weeklyMetricReport.getProjectName());

                Row reportDateRow = sheet.createRow(Constants.WR_EXCEL_REPORT_DATE_ROW);
                Cell reportDateTitleCell = reportDateRow.createCell(Constants.WR_EXCEL_REPORT_DATE_TITLE_CELL);
                reportDateTitleCell.setCellValue(Constants.WR_EXCEL_REPORT_DATE_TITLE);
                reportDateTitleCell.setCellStyle(lockedProjectHeaderColumnCellStyle);

                Cell reportDateValueCell = reportDateRow.createCell(Constants.PS_EXCEL_REPORT_DATE_VALUE_CELL);
                reportDateValueCell.setCellValue(weeklyMetricReport.getReportDate());
                CellStyle reportDateValueCellStyle = workbook.createCellStyle();
                CreationHelper creationHelper = workbook.getCreationHelper();
                short dateFormat = creationHelper.createDataFormat().getFormat("yyyy-MM-dd");
                reportDateValueCellStyle.setDataFormat(dateFormat);
                reportDateValueCell.setCellStyle(reportDateValueCellStyle);

                Row currentWeekTrendRow = sheet.createRow(Constants.WR_EXCEL_CURRENT_WEEK_TREND_ROW);

                Cell trendIdCell = currentWeekTrendRow.createCell(Constants.WR_EXCEL_CURRENT_WEEK_TREND_ID_CELL);
                trendIdCell.setCellValue(String.valueOf(weeklyMetricReport.getTrendId()));

                Cell trendTitleCell = currentWeekTrendRow.createCell(Constants.WR_EXCEL_CURRENT_WEEK_TREND_TITLE_CELL);
                trendTitleCell.setCellValue(Constants.WR_EXCEL_CURRENT_WEEK_TREND_TITLE);
                trendTitleCell.setCellStyle(lockedProjectHeaderColumnCellStyle);

                Cell trendValueCell = currentWeekTrendRow.createCell(Constants.WR_EXCEL_CURRENT_WEEK_TREND_VALUE_CELL);
                trendValueCell.setCellValue(Objects.nonNull(weeklyMetricReport.getCurrentWeekOverallStatus())
                        ? weeklyMetricReport.getCurrentWeekOverallStatus().name() : "NA");

                String[] headersRow = Constants.WR_EXCEL_HEADER_TITLES;
                Row headerRow = sheet.createRow(Constants.WR_EXCEL_HEADER_ROW);
                headerRow.setRowStyle(lockedColumnCellStyle);
                for (int i = 0; i < headersRow.length; i++) {
                    Cell cell = headerRow.createCell(i);
                    cell.setCellValue(headersRow[i]);
                }

                if (!CollectionUtils.isEmpty(weeklyMetricReport.getWeeklyMetrics())) {
                    int weeklyMetricReportRowIndex = Constants.WR_EXCEL_HEADER_ROW + 1;
                    for (int rowIndex = 0; rowIndex < weeklyMetricReport.getWeeklyMetrics().size(); rowIndex++, weeklyMetricReportRowIndex++) {
                        Row row = sheet.createRow(weeklyMetricReportRowIndex);

                        WeeklyMetric weeklyMetric = weeklyMetricReport.getWeeklyMetrics().get(rowIndex);

                        //weekly metric id
                        Cell weeklyMetricIdCell = row.createCell(Constants.WR_EXCEL_WEEKLY_MID_VALUE_CELL, CellType.STRING);
                        weeklyMetricIdCell.setCellValue(String.valueOf(weeklyMetric.getId()));

                        //account metric id
                        Cell accountMetricIdCell = row.createCell(Constants.WR_EXCEL_AM_ID_VALUE_CELL, CellType.STRING);
                        accountMetricIdCell.setCellValue(String.valueOf(weeklyMetric.getAccountMetricId()));

                        //project metric id
                        Cell projectMetricIdCell = row.createCell(Constants.WR_EXCEL_PM_ID_VALUE_CELL, CellType.STRING);
                        projectMetricIdCell.setCellValue(String.valueOf(weeklyMetric.getProjectMetricId()));

                        //category type
                        Cell categoryTypeCell = row.createCell(Constants.WR_EXCEL_TYPE_VALUE_CELL, CellType.STRING);
                        categoryTypeCell.setCellValue(Objects.nonNull(weeklyMetric.getCategoryType())
                                ? weeklyMetric.getCategoryType().name() : "");

                        //category name
                        Cell categoryNameCell = row.createCell(Constants.WR_EXCEL_CATEGORY_VALUE_CELL, CellType.STRING);
                        categoryNameCell.setCellValue(weeklyMetric.getCategoryName());

                        //indicator
                        Cell indicatorCell = row.createCell(Constants.WR_EXCEL_INDICATOR_VALUE_CELL, CellType.STRING);
                        indicatorCell.setCellValue(weeklyMetric.getIndicatorName());

                        //target
                        Cell targetCell = row.createCell(Constants.WR_EXCEL_TARGET_VALUE_CELL, CellType.STRING);
                        targetCell.setCellValue(String.valueOf(weeklyMetric.getTarget()));

                        //actual
                        Cell actualCell = row.createCell(Constants.WR_EXCEL_ACTUAL_VALUE_CELL, CellType.STRING);
                        actualCell.setCellValue(String.valueOf(weeklyMetric.getActual()));

                        //remarks
                        Cell remarksCell = row.createCell(Constants.WR_EXCEL_REMARK_VALUE_CELL, CellType.STRING);
                        remarksCell.setCellValue(StringUtils.isNotEmpty(weeklyMetric.getRemark())
                                ? weeklyMetric.getRemark().replace("|", "\n") : "");
                    }
                }
                sheet.setColumnHidden(0, true);
                sheet.setColumnHidden(1, true);
                sheet.setColumnHidden(2, true);

                // Set auto column width
                for (int columnIndex = 0; columnIndex <= Constants.PS_EXCEL_REMARKS_VALUE_CELL; columnIndex++) {
                    sheet.autoSizeColumn(columnIndex);
                }
                workbook.write(outputStream);
            }

        }
        return outputStream.toByteArray();
    }

    @Override
    public WeeklyMetricReport uploadWeeklyReportExcel(MultipartFile file) throws IOException {
        WeeklyMetricReport weeklyMetricReport = new WeeklyMetricReport();
        List<WeeklyMetric> weeklyMetrics = new ArrayList<>();
        try (InputStream inputStream = file.getInputStream()) {

            Workbook workbook = new XSSFWorkbook(inputStream);
            Sheet sheet = workbook.getSheetAt(0);
            //Trend row
            Row trendRow = sheet.getRow(Constants.WR_EXCEL_CURRENT_WEEK_TREND_ROW);
            weeklyMetricReport.setTrendId((Long) StringToDataTypeConverter.convertStringToDataType(trendRow.getCell(Constants.WR_EXCEL_CURRENT_WEEK_TREND_ID_CELL).getStringCellValue(), Constants.LONG));
            weeklyMetricReport.setCurrentWeekOverallStatus(Status.valueOf(trendRow.getCell(Constants.WR_EXCEL_CURRENT_WEEK_TREND_VALUE_CELL).getStringCellValue()));

            //ReportDate row
            Row reportDateRow = sheet.getRow(Constants.WR_EXCEL_REPORT_DATE_ROW);
            LocalDate reportDate = reportDateRow.getCell(Constants.WR_EXCEL_REPORT_DATE_VALUE_CELL).getLocalDateTimeCellValue().toLocalDate();
            weeklyMetricReport.setReportDate(reportDate);

            //Project row
            Row projectRow = sheet.getRow(Constants.WR_EXCEL_PROJECT_NAME_ROW);
            weeklyMetricReport.setProjectId((Long) StringToDataTypeConverter.convertStringToDataType(projectRow.getCell(Constants.WR_EXCEL_PROJECT_ID_CELL).getStringCellValue(), Constants.LONG));
            weeklyMetricReport.setProjectName(projectRow.getCell(Constants.WR_EXCEL_PROJECT_NAME_VALUE_CELL).getStringCellValue());

            int lastRowNum = sheet.getLastRowNum();
            int weeklyMetricRowIndex = Constants.WR_EXCEL_HEADER_ROW + 1;

            for (int i = weeklyMetricRowIndex; i <= lastRowNum; i++) {
                Row row = sheet.getRow(i);
                if (Objects.nonNull(row)) {
                    weeklyMetrics.add(
                            WeeklyMetric.builder()
                                    .id((Long) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.WR_EXCEL_WEEKLY_MID_VALUE_CELL).getStringCellValue(), Constants.LONG))
                                    .accountMetricId((Long) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.WR_EXCEL_AM_ID_VALUE_CELL).getStringCellValue(), Constants.LONG))
                                    .projectMetricId((Long) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.WR_EXCEL_PM_ID_VALUE_CELL).getStringCellValue(), Constants.LONG))
                                    .categoryType(CategoryType.valueOf(row.getCell(Constants.WR_EXCEL_TYPE_VALUE_CELL).getStringCellValue()))
                                    .categoryName(row.getCell(Constants.WR_EXCEL_CATEGORY_VALUE_CELL).getStringCellValue())
                                    .indicatorName(row.getCell(Constants.WR_EXCEL_INDICATOR_VALUE_CELL).getStringCellValue())
                                    .remark(delimitRemarks(row.getCell(Constants.WR_EXCEL_REMARK_VALUE_CELL).getStringCellValue()))
                                    .target((Long) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.WR_EXCEL_TARGET_VALUE_CELL).getStringCellValue(), Constants.LONG))
                                    .actual((Long) StringToDataTypeConverter.convertStringToDataType(row.getCell(Constants.WR_EXCEL_ACTUAL_VALUE_CELL).getStringCellValue(), Constants.LONG))
                                    .build()
                    );
                }
            }

        }
        if (!CollectionUtils.isEmpty(weeklyMetrics)) {
            weeklyMetricReport.setWeeklyMetrics(weeklyMetrics);
            //return metricsService.updateWeeklyReportForProject(weeklyMetricReport);
        }
        return weeklyMetricReport;
    }

    public String delimitRemarks(String remarks) {
        return StringUtils.isNotEmpty(remarks) ? remarks.replaceAll("\n", "|") : remarks;
    }
}
