package com.altimetrik.pmis.service.impl;

import com.altimetrik.pmis.core.entity.CategoryEntity;
import com.altimetrik.pmis.core.entity.IndicatorEntity;
import com.altimetrik.pmis.mapper.IndicatorMapper;
import com.altimetrik.pmis.model.Indicator;
import com.altimetrik.pmis.repository.CategoryRepository;
import com.altimetrik.pmis.repository.IndicatorsRepository;
import com.altimetrik.pmis.service.IndicatorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Optional;

@Service
public class IndicatorServiceImpl implements IndicatorService {
    private final IndicatorsRepository indicatorsRepository;
    private final IndicatorMapper indicatorMapper;
    private CategoryRepository categoryRepository;

    @Autowired
    public IndicatorServiceImpl(IndicatorsRepository indicatorsRepository, CategoryRepository categoryRepository) {
        this.indicatorsRepository = indicatorsRepository;
        this.indicatorMapper = IndicatorMapper.INSTANCE;
        this.categoryRepository = categoryRepository;
    }

    @Override
    @Transactional
    public Indicator save(Indicator indicator) {
        Optional<IndicatorEntity> indicatorEntityOptional = indicatorsRepository.findByNameAndProjectTypeAndCategoryCategoryNameAndCategoryType(indicator.getName(), indicator.getProjectType(),
                indicator.getCategory().getCategoryName(), indicator.getCategory().getType());
        if (indicatorEntityOptional.isPresent()) {
            return indicatorMapper.entityToDomain(indicatorEntityOptional.get());
        }
        Optional<CategoryEntity> categoryEntityOptional = categoryRepository.findByCategoryNameAndType(indicator.getCategory().getCategoryName(), indicator.getCategory().getType());

        return indicatorMapper.entityToDomain(indicatorsRepository.save(indicatorMapper.indicatorDomainToEntity(indicator, categoryEntityOptional)));
    }

    @Override
    public Indicator get(Long id) {
        Optional<IndicatorEntity> indicatorEntityOptional = indicatorsRepository.findById(id);
        if (!indicatorEntityOptional.isPresent()) {
            //TODO:: throw exception
        }
        return indicatorMapper.entityToDomain(indicatorEntityOptional.get());
    }

    @Override
    public List<Indicator> getAll() {
        List<IndicatorEntity> indicatorEntities = indicatorsRepository.findAll();
        if (!CollectionUtils.isEmpty(indicatorEntities)) {
            //TODO:: throw exception
        }
        return indicatorMapper.entityToDomain(indicatorEntities);
    }
}
